#!/bin/bash
# rRNA pipeline beginning with trimmed sequences
# This script is run from a directory above the directory trimmed_seqs containing the trimmed sequences to be processed.

# Configure paths:

# This configuration assumes that programs are installed as directed on john-quensen.com.
# The paths may differ for other installations.
# RDPToolsDir=/usr/local/RDPTools
# infernalDir=/usr/lib/infernal
# cmModelDir=/usr/local/fungene_pipeline/resources/RRNA_16S_BACTERIA
# usearch=/usr/local/bin/usearch8.1
# FastTree=/usr/bin/fasttree

# This configuration is correct for MSU's HPCC.
RDPToolsDir=/mnt/research/rdp/public/RDPTools
infernalDir=/mnt/research/rdp/public/thirdParty/infernal-1.1/src
cmModelDir=/mnt/research/rdp/public/fungene_pipeline/resources/RRNA_16S_BACTERIA
module load USEARCH
module load FastTree
usearch=usearch
FastTree=FastTree

# For both configurations:
refSeqDir=$HOME/resources
JAVA_HEAP_SIZE=2g

## Remove chimeras with uchime in reference mode. Non-chimeric sequences are written to the sub-directory non-chimeric_seqs.
mkdir non-chimeric_seqs
cd trimmed_seqs
for f in $(ls ./*.fastq)
do
     $usearch -uchime_ref $f -db $refSeqDir/rdp_gold.fa -nonchimeras ../non-chimeric_seqs/${f/fastq/}fasta -strand plus
done

## Skip removing chimeras. Instead copy sequences to the sub-directory non-chimeric_seqs.
#mkdir non-chimeric_seqs
#cd trimmed_seqs
#for f in $(ls ./*.fastq)
#do
#     java -Xmx$JAVA_HEAP_SIZE -jar /usr/local/RDPTools/ReadSeq.jar to-fasta $f > ../non-chimeric_seqs/${f/fastq/}fasta
#done

# Align the chimera-free sequences for each sample. Put the results in sub-directory aligned_nc.
cd ../
mkdir aligned_nc
cd non-chimeric_seqs
for f in $(ls *.fasta)
do
	$infernalDir/cmalign -g --noprob --outformat Stockholm --dnaout -o ../aligned_nc/${f/fasta/}stk $cmModelDir/model.cm $f
done

# Merge the alignments.
cd ../
mkdir merged_alignment
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/AlignmentTools.jar alignment-merger aligned_nc merged_alignment/merged_aligned.fasta

# Cluster, 3 steps.
mkdir cluster
cd cluster
# 1) Dereplicate
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar derep -m '#=GC RF' -o derep.fa all_seqs.ids all_seqs.samples ../aligned_nc/*.stk

# 2) Calculate the distance matrix.
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar dmatrix --id-mapping all_seqs.ids --in derep.fa --outfile derep_matrix.bin -l 200 --dist-cutoff 0.1

# 3) Cluster
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar cluster --dist-file derep_matrix.bin --id-mapping all_seqs.ids --sample-mapping all_seqs.samples --method complete --outfile all_seq_complete.clust

# Convert cluster file to biom format.
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar cluster-to-biom all_seq_complete.clust 0.03 > all_seq_complete.clust.biom

# Get representative sequences.
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar rep-seqs -c --id-mapping all_seqs.ids --one-rep-per-otu all_seq_complete.clust 0.03 ../merged_alignment/merged_aligned.fasta

# Add classification to the biom file.
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/classifier.jar classify -c 0.5 -f biom -m all_seq_complete.clust.biom -o all_seq_complete_classified.biom all_seq_complete.clust_rep_seqs.fasta

# Unalign representative sequences and remove descriptions. 
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar to-unaligned-fasta all_seq_complete.clust_rep_seqs.fasta | cut -f1 -d ' ' > unaligned_short_names.fasta

# Tree the representative sequences.
# First, extract the comparable (model) positions for tree building.
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar derep -f -o all_seq_complete.clust_rep_seqs_model_only.fasta rep_seqs.ids rep_seqs.sample all_seq_complete.clust_rep_seqs.fasta

# Build the tree.
export OMP_NUM_THREADS=1
$FastTree -nt -gtr < all_seq_complete.clust_rep_seqs_model_only.fasta > my_expt_tree.nwk

# Package results for phyloseq.
zip results4phyloseq.zip all_seq_complete_classified.biom unaligned_short_names.fasta my_expt_tree.nwk
mv results4phyloseq.zip ../

